"""
### CODE OWNERS: Demerrick Moton
### OBJECTIVE:
    Create high-level constructs for implementing and managing ETL processes
### DEVELOPER NOTES:
"""
import logging
import os

from google.cloud import bigquery

logging.basicConfig(
    format="%(asctime)s - %(message)s", level=os.environ.get("LOGLEVEL", "INFO")
)
LOGGER = logging.getLogger(__name__)

SPARK_BQ_TYPEMAP = {
    "StringType": "STRING",
    "VarcharType": "STRING",
    "CharType": "STRING",
    "ByteType": "BYTES",
    "ShortType": "INTEGER",
    "IntegerType": "INTEGER",
    "LongType": "BIGNUMERIC",
    "FloatType": "FLOAT",
    "DoubleType": "FLOAT",
    "DecimalType": "FLOAT",
    "BooleanType": "BOOLEAN",
    "TimestampType": "TIMESTAMP",
    "DateType": "DATE",
}

# ===========================================================


class BigQueryConnector:
    def __init__(self, project_id: str):
        self.project_id = project_id
        self.client = bigquery.Client(project=project_id)

    # def __new__(cls):
    #     if cls._instance is None:
    #         LOGGER.info("Creating the BigQueryConnector")
    #         cls._instance = super(BigQueryConnector, cls).__new__(cls)
    #         # initialization steps
    #     return cls._instance

    def read(self, dataset: str, table: str):
        """Read table from BigQuery given bucket and table names"""
        table = bigquery.TableReference.from_string(
            "canceriq-data-test.{}.{}".format(dataset, table)
        )
        query = """
            SELECT * FROM {}
        """.format(
            table
        )
        query_job = self.client.query(query)
        results = query_job.result()
        pd_df = results.to_dataframe()
        return pd_df

    # load data into bq
