"""
### CODE OWNERS: Boni Nugraha
### OBJECTIVE:
    Make connections with database
### DEVELOPER NOTES:
"""

import logging
import pandas as pd
from sqlalchemy import create_engine

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class DBConnector:
    def __init__(self, instance_ip, db_name, username, password) -> None:
        self.CLOUDSQL_INSTANCE_IP = instance_ip
        self.CLOUDSQL_DB_NAME = db_name
        self.CLOUDSQL_USER = username
        self.CLOUDSQL_PWD = password

        self.engine = create_engine(
            f"postgresql+psycopg2://{self.CLOUDSQL_USER}:{self.CLOUDSQL_PWD}@{self.CLOUDSQL_INSTANCE_IP}/{self.CLOUDSQL_DB_NAME}?client_encoding=utf8"
        )

    def read_schema_df(self, table_name: str) -> pd.DataFrame:
        query = f"SELECT table_name, column_name, is_nullable, data_type FROM information_schema.columns WHERE table_schema='public' and table_name='{table_name}'"
        pdf = pd.read_sql(sql=query, con=self.engine)
        return pdf

    def read_table_query(self, query: str) -> pd.DataFrame:
        """
        Read table using provided query

        Args:
            query (str): Provided query

        Returns:
            pd.DataFrame: Pandas Dataframe
        """
        return pd.read_sql(sql=query, con=self.engine)

    def read_table(
        self, table_name: str, columns: list, limit_clause: str = ""
    ) -> pd.DataFrame:
        """
        Read table using predefined parameterized query

        Args:
            table_name (str): Name of table
            columns (list): Desired columns
            limit_clause (str): Desired row limit

        Returns:
            pd.DataFrame: [description]
        """
        LOGGER.info("Loading Datasets from Postgres...")
        query = f"select {', '.join(columns)} from {table_name}" + limit_clause
        pdf = pd.read_sql(sql=query, con=self.engine)

        return pdf

    def write_table(self, dataframe, table_name):
        LOGGER.info("Write Datasets from Postgres...")
        dataframe.to_sql(table_name, self.engine, index=False, if_exists="replace")
        return True

    def close_connection(self):
        LOGGER.info("Connection to Postgres disposed.")
        self.engine.dispose()
        return True
