"""
### CODE OWNERS: Boni Nugraha
### OBJECTIVE:
    Get credentials from Secret Manager
### DEVELOPER NOTES:
"""

import logging
import json
import hashlib

from google.cloud import secretmanager

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# ===========================================================


class SecretManagerConnector:
    def __init__(self) -> None:
        self.client = secretmanager.SecretManagerServiceClient()

    def read_secrets_string(self, project_id, secret_id, version_id="latest"):
        # Build the resource name of the secret version.
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{version_id}"

        # Access the secret version.
        response = self.client.access_secret_version(name=name)

        # Return the decoded payload.
        return response.payload.data.decode("UTF-8")

    def read_secrets_dictionary(self, project_id, secret_id, version_id="latest"):
        secret_str = self.read_secrets_string(project_id, secret_id, version_id)
        secret_dict = json.loads(secret_str)
        return secret_dict

    def secret_hash(secret_value):
        # return the sha224 hash of the secret value
        return hashlib.sha224(bytes(secret_value, "utf-8")).hexdigest()
